#include "TeaNetworking.hpp"

void TeaNetworking::init(const char *ssid, const char *password)
{
  // networking init
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("");
  Serial.print("Connecting");
  int attempts = 0;
  while (WiFi.status() != WL_CONNECTED)
  {
    /*
    if (attempts++ > 30)
    {
      Serial.println("");
      Serial.println("Failed to connect to network.");
      return;
    }
    */
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());

  // Set time via NTP, as required for x.509 validation
  configTime(3 * 3600, 0, "pool.ntp.org", "time.nist.gov");
  Serial.print("Waiting for NTP time sync: ");
  time_t now = time(nullptr);
  while (now < 8 * 3600 * 2)
  {
    delay(500);
    Serial.print(".");
    now = time(nullptr);
  }
  Serial.println("");
  struct tm timeinfo;
  gmtime_r(&now, &timeinfo);
  Serial.print("Current time: ");
  Serial.print(asctime(&timeinfo));
}

JSONVar TeaNetworking::httpsGETRequest(String serverName)
{
  WiFiClientSecure client;
  String payload = "{}";
  // wait for WiFi connection
  if (WiFi.status() == WL_CONNECTED)
  {
    // Create a list of certificates with the server certificate
    X509List cert(IRG_Root_X1);
    client.setTrustAnchors(&cert);
    HTTPClient http;
    Serial.print("[HTTPS] begin...\n");
    if (http.begin(client, serverName))
    { // HTTPS
      Serial.print("[HTTPS] GET...\n");
      // start connection and send HTTP header
      int httpCode = http.GET();
      // httpCode will be negative on error
      if (httpCode > 0)
      {
        // HTTP header has been send and Server response header has been handled
        Serial.printf("[HTTPS] GET... code: %d\n", httpCode);
        // file found at server
        if (httpCode == HTTP_CODE_OK || httpCode == HTTP_CODE_MOVED_PERMANENTLY)
        {
          payload = http.getString();
          Serial.println(payload);
        }
      }
      else
      {
        Serial.printf("[HTTPS] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
      }
      http.end();
    }
    else
    {
      Serial.printf("[HTTPS] Unable to connect\n");
    }
  }
  return JSON.parse(payload);
}

JSONVar TeaNetworking::httpsPOSTRequest(String serverName, String data)
{
  WiFiClientSecure client;
  String payload = "{}";

  // wait for WiFi connection
  if ((WiFi.status() == WL_CONNECTED))
  {
    // Create a list of certificates with the server certificate
    X509List cert(IRG_Root_X1);
    client.setTrustAnchors(&cert);
    HTTPClient http;
    Serial.print("[HTTPS] begin...\n");
    // http.setFollowRedirects(HTTPC_FORCE_FOLLOW_REDIRECTS);
    if (http.begin(client, serverName))
    { // HTTPS
      // Send HTTPS POST request
      http.addHeader("Content-Type", "application/json");
      int httpCode = http.POST(data);
      // httpCode will be negative on error
      if (httpCode > 0)
      {
        // HTTP header has been send and Server response header has been handled
        Serial.printf("[HTTPS] POST... code: %d\n", httpCode);
        payload = http.getString();
        Serial.println(payload);
      }
      else
      {
        Serial.printf("[HTTPS] POST... failed, error: %s\n", http.errorToString(httpCode).c_str());
      }
      http.end();
    }
    else
    {
      Serial.printf("[HTTPS] Unable to connect\n");
    }
  }
  return JSON.parse(payload);
}

String TeaNetworking::createTeaData(
    String teaName,
    String rfidCode,
    int waterTemp,
    int steepingSeconds)
{
  JSONVar teaData;
  teaData["teaname"] = teaName;
  teaData["rfidcode"] = rfidCode;
  teaData["watertemp"] = waterTemp;
  teaData["steepingseconds"] = steepingSeconds;
  return JSONVar::stringify(teaData);
}

String TeaNetworking::createTeaData(TeaData tea)
{
  JSONVar teaData;
  teaData["teaname"] = tea.m_teaName;
  teaData["rfidcode"] = tea.m_rfidCode;
  teaData["watertemp"] = tea.m_waterTemp;
  teaData["steepingseconds"] = tea.m_steepingSeconds;
  return JSONVar::stringify(teaData);
}